@@ -53,11 +53,14 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/xpm.h>
+
+// wroot.h must be included after all othe x11 related includes
+#include "vroot.h"
+
 #include <ctype.h>
 #include <assert.h>
 
 #include "xfishtank.h"
-#include "vroot.h"
 #include "bubbles.h"
 #include "debug.h"
 #include "transwindow.h"
@@ -66,6 +69,8 @@
 #include "ixpm.h"
 #include "ui.h"
 #include "xdo.h"
+#include "windows.h"
+
 
 
 #ifdef DOUBLE_BUFFER
@@ -92,6 +97,7 @@ typedef struct _fish {
    float x;
    float y;
    int   direction;        // direction: 0: r->l
+   float y_direction;      // vertical direction: -1: to the top, 0: no movement, 1: to bottom
    int   frame;            // animation frames: 0 .. NUM_FRAMES
    int   type; 
    float fstep;            // increment in placement
@@ -108,8 +114,6 @@ typedef struct _fishtype {
 
 static int      binc[] = { 0, 64, 56, 48, 40, 32, 24, 16, 8 }; /* bubble increment and yes check tables */
 static int      DoubleBuf = 0;               /* Should we use double buffering */
-static int      width;                       /* width of initial window in pixels */
-static int      height;                      /* height of initial window in pixels */
 static int      screen;                      /* Default screen of this display */
 static int      inxscreensaver = 0;          /* are we running in xscreensaver? */
 static float    smooth = 0.02;               /* smoothness increment multiplier */
@@ -119,16 +123,14 @@ static Window   root_window;
 static fishtype xfish[NUM_FISH][2][NUM_FRAMES];        /* [type][left/right][frame] */
 
 static Pixmap xbubbles[9];              /* bubbles bitmaps (1 to 8, by size in pixels) */
-static Window wid;                      /* aquarium window */
-static Window wid1;
 static Window UserWindow = 0;
 static Pixel  white;
 static Pixel  black;
 static Pixel  bcolor;
 static Pixel  bgcolor;
 static GC bgc;
-static int xfishtank_trans = 0;
-static XdbeBackBuffer backbuf = 0;
+//static XdbeBackBuffer backbuf = 0;
+static Drawable backbuf = 0;
 static int  WantXdbe = 1;
 static const int maxanimtime = 20;
 static int ForceRoot = 0;
@@ -136,30 +138,31 @@ static int HaltedByInterrupt = 0;
 static int Done = 0;
 
 static Pixel AllocNamedColor(Display *display, char *colorName, Pixel dfltPix);
-static char *display_name=NULL;
-static int  do_move(void *dummy);
-static int  do_testfish(void *dummy);
-static int  do_write_flags(void *dummy);
-static void erasebubble(bubble *b);
-static void erasefish(fish *f);
-static void init_pixmap(void);
-static void init_signals(void);
-static void initialize(void);
-static void move_fish(void);
-static void movefish(fish *f);
-static void new_bubble(bubble *b, int init);
-static void new_fish(fish *f0, int init);
-static void parse(int argc, char **argv);
-static void putbubble(bubble *b, unsigned long c);
-static void remove_all_bubbles(void);
-static void remove_all_fishes(void);
-static void setanimtime(fish*f);
-static void step_bubbles(void);
-static void Usage(void);
-static void Thanks(void);
-static void SigHandler(int signum);
-static void print_changelog(void);
-static void selfrep(void);
+static char  *display_name=NULL;
+static int   do_move(void *dummy);
+static int   do_testfish(void *dummy);
+static int   do_write_flags(void *dummy);
+static void  erasebubble(bubble *b);
+static void  erasefish(fish *f);
+static void  init_pixmap(void);
+static void  init_signals(void);
+static void  initialize(void);
+static void  move_fish(void);
+static void  draw_fish(fish *f);
+static void  new_bubble(bubble *b, int init);
+static void  new_fish(fish *f0, int init);
+static void  parse(int argc, char **argv);
+static void  putbubble(bubble *b, unsigned long c);
+static void  remove_all_fishes(void);
+static float random_y_direction(void);
+static float random_fstep(void);
+static void  setanimtime(fish*f);
+static void  step_bubbles(void);
+static void  Usage(void);
+static void  Thanks(void);
+static void  SigHandler(int signum);
+static void  print_changelog(void);
+static void  selfrep(void);
 
 
 Display *Dpy = NULL;
@@ -170,9 +173,19 @@ int     flimit = 10;                 /* fish limit */
 char   *bcolorstring  = NULL;
 char   *bgcolorstring = NULL;
 float   speedfactor;
+int     allworkspaces = 1;
+int     monitor = -1;
 char   *DesktopSession;
 xdo_t  *myxdo;
 int     nomenu = 0;
+GtkWidget *gtkwin;
+int      width;                       /* width of initial window in pixels */
+int      height;                      /* height of initial window in pixels */
+int      fullwidth;                   /* width of full screen window in pixels */
+int      fullheight;                  /* height of full screen window in pixels */
+int      xfishtank_trans = 0;
+Window   wid;                      /* aquarium window */
+Window   wid1;
 
 
 
@@ -217,16 +230,29 @@ void parse(int argc, char *argv[])
 	    free(bgcolorstring);
 	 bgcolorstring = strdup(argv[p]);
       } 
-      else if (!strcmp(argv[p],"-double")) 
-      {
+      /*
+	 else if (!strcmp(argv[p],"-double")) 
+	 {
 	 CHECK;
 	 WantXdbe = (strtod(argv[p],NULL)>0);
+      // without double buffering, xfishtank misbehaves
       } 
+      */
       else if (!strcmp(argv[p],"-speed")) 
       {
 	 CHECK;
 	 speedfactor = (strtod(argv[p],NULL)/100.0);
       } 
+      else if (!strcmp(argv[p],"-allworkspaces"))
+      {
+	 CHECK;
+	 allworkspaces = strtod(argv[p],NULL);
+      }
+      else if (!strcmp(argv[p],"-monitor"))
+      {
+	 CHECK;
+	 monitor = strtod(argv[p],NULL);
+      }
       else if (!strcmp(argv[p],"-h")) 
       {
 	 Usage();
@@ -253,6 +279,7 @@ void parse(int argc, char *argv[])
       {
 	 ForceRoot = 1;
       } 
+      // this is used by xscreensaver:
       else if (!strcmp(argv[p],"-window-id")) 
       {
 	 CHECK;
@@ -295,6 +322,9 @@ void set_defaults()
       free(bgcolorstring);
    bgcolorstring = strdup("darkblue");
    speedfactor   = 1.0;
+   allworkspaces = 1;
+   monitor       = -1;
+   set_monitor(monitor);
 }
 
 void Usage()
@@ -306,15 +336,15 @@ void Usage()
    printf("        -bc color  color of bubbles (default \"lightblue\")\n");
    printf("        -bgc color color of background (default \"darkblue\")\n");
    printf("                   only when running under xscreensaver or in a non-compositing WM\n");
-   printf("        -double n  1: double buffering, 0: do not use (default: 1)\n");
+   // printf("        -double n  1: double buffering, 0: do not use (default: 1)\n");
    printf("        -speed     speed of fishes (default: 100)\n");
    printf("        -defaults  all options to default\n");
    printf("        -nomenu    minimize menu (NOTE: in xmonad e.d. the 'hide menu' button will not work)\n");
    printf("        -display   DISPLAY to draw on (default \"\")\n");
    printf("        -root      use root-window or xscreensaver-provided window to draw in\n");
    printf("        --root     same as -root\n");
-   printf("        -window-id window to draw on\n");
-   printf("        --window-id same as -window-id\n");
+   // printf("        -window-id window to draw on\n");
+   // printf("        --window-id same as -window-id\n");
    printf("        -h         show this info\n");
    printf("        -v         show version\n");
    printf("        -changelog show ChangeLog and exit\n");
@@ -350,24 +380,50 @@ void erasefish(fish *f)
     */
    //XClearArea(Dpy, wid, f->x, f->y, rwidth[f->type], rheight[f->type], False);
    fishtype *ft = &xfish[f->type][f->direction][f->frame];
-   XClearArea(Dpy, wid, f->x, f->y, ft->w, ft->h, False);
+   if(xfishtank_trans)
+   {
+      XClearArea(Dpy, wid, f->x, f->y, ft->w, ft->h, False);
+   }
+   else
+   {
+      XClearArea(Dpy, wid, monitor_x+f->x, monitor_y+f->y, ft->w, ft->h, False);
+   }
 }
 
 
-/*
- * This function can only be called if DoClipping is True.  It is used to
- * move a clipmasked fish.  First the area under the fish is cleared,
- * and then the new fish is masked in.
- * The parameters x, y, amd d are from the old fish that is being
- * erased before the new fish is drawn.
- */
-void movefish(fish *f)
+// draw fish
+void draw_fish(fish *f)
 {
    fishtype *ft = &xfish[f->type][f->direction][f->frame];
-   XSetClipOrigin(Dpy, ft->gc, f->x, f->y);
-   XCopyArea(Dpy, ft->pix, wid,ft->gc,0,0, ft->w, ft->h, f->x, f->y);
+   if(xfishtank_trans)
+   {
+      XSetClipOrigin(Dpy, ft->gc, f->x, f->y);
+      // we have a separate window for each monitor
+      XCopyArea(Dpy, ft->pix, wid,ft->gc, 0, 0, ft->w, ft->h, f->x, f->y);
+   }
+   else
+   {
+      // we don't have a separate window for each monitor
+      XSetClipOrigin(Dpy, ft->gc, f->x, f->y);
+      if (screen == -1) // full screen mode
+      {
+	 XCopyArea(Dpy, ft->pix, wid,ft->gc, 0, 0, ft->w, ft->h, f->x, f->y);
+      }
+      else
+      {
+	 XSetClipOrigin(Dpy, ft->gc, monitor_x + f->x, monitor_y + f->y);
 
-   P("movefish: %#lx %d %d %d %d %d %d %d\n",wid,f->type,f->direction,f->frame,(int)f->x,(int)f->y,ft->w,ft->h);
+	 //if (f->x + ft->w < width)
+	 //  XCopyArea(Dpy, ft->pix, wid,ft->gc, 0, 0, ft->w,        ft->h, f->x, f->y);
+	 //else
+	 //  XCopyArea(Dpy, ft->pix, wid,ft->gc, 0, 0, width - f->x, ft->h, f->x, f->y);
+	 P("myx myy: %d %d %d %d\n",monitor_x,monitor_y,(int)(monitor_x+f->x),(int)(monitor_y+f->y));
+	 XCopyArea(Dpy, ft->pix, wid,ft->gc, 0, 0, width - f->x, height - f->y, monitor_x+f->x, monitor_y+f->y);
+      }
+   }
+
+
+   P("draw_fish: %#lx %d %d %d %d %d %d %d\n",wid,f->type,f->direction,f->frame,(int)f->x,(int)f->y,ft->w,ft->h);
    f->animtime --;
    if (f->animtime <= 0)
    {
@@ -383,7 +439,11 @@ void setanimtime(fish*f)
 
 void erasebubble(bubble *b)
 {
-   XClearArea(Dpy, wid, b->x, b->y, b->size, b->size, 0);
+   if(xfishtank_trans)
+      XClearArea(Dpy, wid, b->x, b->y, b->size, b->size, 0);
+   else
+      XClearArea(Dpy, wid, monitor_x+b->x, monitor_y+b->y, b->size, b->size, 0);
+
    P("erasebubble: %d %d %d\n",b->x, b->y, b->size);
 }
 
@@ -395,11 +455,21 @@ void putbubble(bubble *b, unsigned long c)
    int s = b->size;
    gcv.foreground = c;
    gcv.clip_mask = xbubbles[s];
-   gcv.clip_x_origin = b->x;
-   gcv.clip_y_origin = b->y;
-   XChangeGC(Dpy, bgc, GCForeground | GCClipMask | GCClipXOrigin | GCClipYOrigin, &gcv);
-   XFillRectangle(Dpy, wid, bgc, b->x, b->y, s, s);
-   P("putbubble: %d %d\n", b->x, b->y);
+   if(xfishtank_trans)
+   {
+      gcv.clip_x_origin = b->x;
+      gcv.clip_y_origin = b->y;
+      XChangeGC(Dpy, bgc, GCForeground | GCClipMask | GCClipXOrigin | GCClipYOrigin, &gcv);
+      XFillRectangle(Dpy, wid, bgc, b->x, b->y, s, s);
+   }
+   else
+   {
+      gcv.clip_x_origin = monitor_x + b->x;
+      gcv.clip_y_origin = monitor_y + b->y;
+      XChangeGC(Dpy, bgc, GCForeground | GCClipMask | GCClipXOrigin | GCClipYOrigin, &gcv);
+      XFillRectangle(Dpy, wid, bgc, monitor_x + b->x, monitor_y + b->y, s, s);
+   }
+   P("putbubble: %d %d\n", monitor_y + b->x, monitor_y + b->y);
 }
 
 
@@ -506,21 +576,26 @@ void initialize()
    root_window = VirtualRootWindowOfScreen(DefaultScreenOfDisplay(Dpy));
 
    XGetWindowAttributes(Dpy, root_window, &winfo);
-   width = winfo.width;
-   height = winfo.height;
+   fullwidth = winfo.width;
+   fullheight = winfo.height;
+   width = fullwidth;
+   height = fullheight;
+
 
    DoubleBuf=0;
 
    int x,y;
    unsigned int w,h,b,depth;
    Window root, searchWin;
-   GtkWidget   *gtkwin;
    XGetGeometry(Dpy,root_window,&root,
 	 &x, &y, &w, &h, &b, &depth);
    gtkwin = gtk_window_new(GTK_WINDOW_TOPLEVEL); 
-   gtk_window_set_title             (GTK_WINDOW(gtkwin),"Xfishtank-A");
-   gtk_window_set_skip_taskbar_hint (GTK_WINDOW(gtkwin),TRUE);
-   gtk_window_set_skip_pager_hint   (GTK_WINDOW(gtkwin),TRUE);
+   gtk_window_set_title             (GTK_WINDOW(gtkwin),"XfishtanK");
+
+   //gtk_window_set_skip_taskbar_hint (GTK_WINDOW(gtkwin),TRUE);
+   //gtk_window_set_skip_pager_hint   (GTK_WINDOW(gtkwin),TRUE);
+   gtk_window_set_decorated(GTK_WINDOW(gtkwin), FALSE);
+   gtk_window_set_type_hint(GTK_WINDOW(gtkwin), GDK_WINDOW_TYPE_HINT_POPUP_MENU);
 
    if (ForceRoot)
    {
@@ -529,6 +604,7 @@ void initialize()
       if (getenv("XSCREENSAVER_WINDOW"))
       {
 	 searchWin = strtol(getenv("XSCREENSAVER_WINDOW"),NULL,0);
+	 monitor = -1;
 	 inxscreensaver = 1;
 	 nomenu = 1;
       }
@@ -539,13 +615,28 @@ void initialize()
    }
    else
    {
+#if 0 
       int HaveTrans = make_trans_window(gtkwin,
 	    1 /*fullscreen*/,
-	    1 /*sticky*/,
+	    allworkspaces /*sticky*/,
 	    1 /* below*/,
 	    1 /* dock*/ , 
 	    NULL,
 	    &searchWin);
+#else
+      int HaveTrans = make_trans_window(
+	    Dpy,
+	    gtkwin,
+	    monitor,
+	    allworkspaces /*sticky*/,
+	    1 /* below*/,
+	    0 /* dock */,
+	    NULL /* gdkwindow */,
+	    &searchWin,
+	    NULL, // wantx
+	    NULL  // wanty
+	    );
+#endif
       if (HaveTrans)
       {
 	 xfishtank_trans = 1;
@@ -643,6 +734,8 @@ void initialize()
       else
 	 printf("This is an existing window, probably the root window.\n");
    }
+   printf("Size: %dx%d\n",width,height);
+   fflush(NULL);
    XClearWindow(Dpy,wid1);
 
    // drawing on wid, XClearWindow, XSetWindowBackground on wid1
@@ -665,7 +758,7 @@ void new_bubble(bubble *b, int init)  // init=0: postions at bottom, else random
    b->size = s = 1.0 + 8*drand48();
    if ((b->bstep = smooth * height / (float) binc[s]) == 0)
       b->bstep = 1;
-   P("newbubble %d %d\n",b->x, b->y);
+   P("newbubble %f %f %d %d\n",b->x, b->y, monitor_x, width);
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
@@ -726,8 +819,7 @@ void new_fish(fish *f0, int init)  // init 0: position at edge, else position at
    for (i = 0, collide = 1; (i < 16) && (collide); i++) 
    {
       f->y = (height - ft->h) * drand48();
-      if ((f->fstep = smooth * width / (8.0 * (1.0 + 8*drand48()))) == 0) 
-	 f->fstep = 1;
+      f->fstep = random_fstep();
    }
    if (drand48() < 0.5) 
    {
@@ -746,11 +838,28 @@ void new_fish(fish *f0, int init)  // init 0: position at edge, else position at
 	 f->x = -ft->w;
    }
 
+   f->y_direction = random_y_direction();
+
    f->frame = 0;
    setanimtime(f);
    P("newfish %d %d\n",(int)f->x,(int)f->y);
 }
 
+float random_y_direction()
+{
+   return 2*drand48()-1;
+}
+
+float random_fstep()
+{
+   float f;
+   if ((f = smooth * width / (8.0 * (1.0 + 8*drand48()))) < 0.5) 
+      f = 0.5;
+   return f;
+}
+
+
+
 void remove_all_fishes()
 {
    int i;
@@ -774,13 +883,22 @@ void remove_all_bubbles()
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 void move_fish()
 {
-   int i, y, done = 0;
+   int i, done = 0;
    fish *f;
 
    for (i = 0; i < flimit; i++) 
    {
       f = &finfo[i];
       fishtype *ft = &xfish[f->type][f->direction][f->frame];
+
+      //if (f->y > height - ft-> h || f->x > width)
+      if (f->x > width)
+      {
+	 erasefish(f);
+	 new_fish(f,1);
+	 continue;
+      }
+
       if (f->direction == 0) 
 	 done = ((f->x -= speedfactor*f->fstep) < -ft->w);
       else if (f->direction == 1) 
@@ -788,25 +906,22 @@ void move_fish()
 
       if(1) // spontaneous change direction now and then
 	 if (f->x > ft->w && f->x < width - 2*ft->w && drand48() < 0.001) 
-	 {
 	    f->direction = 1-f->direction;
-	    P("direction: %d\n",f->direction);
-	 }
+
+      if(1) // spontaneous change vertical movement now and then
+	 if(drand48() < 0.004)
+	    f->y_direction = random_y_direction();
+
+      if(1) // spontaneous change speed now and then
+	 if(drand48() < 0.004)
+	    f->fstep = random_fstep();
+
       if (1) 
       {
 	 if (!done) 
 	 {
-	    double r = drand48();
-	    if (r < 0.25) 
-	       y = f->fstep / 4;
-	    else if (r > 0.75) 
-	       y = f->fstep / -4;
-	    else 
-	       y = 0;
-
-	    if (y) 
-	       f->y += y;
-	    movefish(f);
+	    f->y += 0.5*f->fstep * f->y_direction;
+	    draw_fish(f);
 	 } 
 	 else
 	 {
@@ -883,10 +998,11 @@ int do_testfish(void *dummy)
 
 void create_fishes(int n)
 {
+   P("create_fish %d\n",n);
    int i;
    static int prev = 0;
    flimit = n;
-   finfo = (fish *)realloc(finfo,sizeof(fish)*flimit);
+   finfo = (fish *)realloc(finfo,1+sizeof(fish)*flimit); // 1+: otherwise flimit=0 gives allocation error
    if(finfo == NULL)
    {
       fprintf(stderr,"Realloc error in %s:%d\n",__FILE__,__LINE__);
@@ -902,7 +1018,7 @@ void create_bubbles(int n)
    int i;
    static int prev = 0;
    blimit = n;
-   binfo = (bubble *)realloc(binfo,sizeof(bubble)*blimit);
+   binfo = (bubble *)realloc(binfo,1+sizeof(bubble)*blimit);
    if(binfo == NULL)
    {
       fprintf(stderr,"Realloc error in %s:%d\n",__FILE__,__LINE__);
@@ -977,7 +1093,14 @@ int main(int argc, char *argv[])
    else
       DesktopSession = strdup("unknown");
 
-   srand48(time(NULL));
+   {
+      struct timeval tv;
+      gettimeofday(&tv,NULL);
+      unsigned long x = 1000000*time(NULL) + tv.tv_usec;
+      srand48(x);
+
+      P("TD: srand: %ld %f\n",x,drand48());
+   }
    counter = 0;
    set_defaults();
    ReadFlags();
@@ -985,7 +1108,10 @@ int main(int argc, char *argv[])
    gtk_init(&argc, &argv);
    printf("xfishtank version %s\n",VERSION);
    printf("Detected desktop session: %s\n",DesktopSession);
+
    Dpy = XOpenDisplay(display_name);
+
+
    if(!Dpy)
    {
       char *name = display_name;
@@ -1018,15 +1144,16 @@ int main(int argc, char *argv[])
 
    srand((unsigned) getpid());
 
+   ui(nomenu);
+
    create_bubbles(blimit);
 
    create_fishes(flimit);
 
-   if (!nomenu)
-      ui();
 
    g_timeout_add_full(G_PRIORITY_DEFAULT, 30,   do_move        ,NULL, NULL);
-   g_timeout_add_full(G_PRIORITY_DEFAULT, 500,  do_write_flags ,NULL, NULL);
+   if (inxscreensaver == 0)
+      g_timeout_add_full(G_PRIORITY_DEFAULT, 500,  do_write_flags ,NULL, NULL);
 
    if(0)
       g_timeout_add_full(G_PRIORITY_DEFAULT, 40,   do_testfish ,NULL, NULL);
@@ -1043,6 +1170,7 @@ void print_changelog()
 #include "changelog.inc"
 }
 
+
 #ifdef SELFREP
 static unsigned char tarfile[] = {
 #include "tarfile.inc"
