@@ -27,10 +27,13 @@
 
 #include <gtk/gtk.h>
 #include <math.h>
+#include <X11/extensions/Xinerama.h>
+#include <assert.h>
 #include "ui_xml.h"
 #include "ui.h"
 #include "debug.h"
 #include "xfishtank.h"
+#include "windows.h"
 #include "utils.h"
 #include "fishes.h"
 
@@ -45,6 +48,8 @@
 #endif
 
 int FlagsChanged = 1;
+int HaveXinerama;
+int monitor_x = 0, monitor_y = 0;
 
 static int    human_action = 1;   /* is it a human who is playing with the buttons? */
 
@@ -64,11 +69,17 @@ static GtkWidget       *id_logo3;
 static GtkWidget       *id_bubble_color;
 static GtkWidget       *id_bg_color;
 static GtkWidget       *id_bubblesvalue;
+static GtkWidget       *id_screen;
+static GtkWidget       *id_allworkspaces;
+static GtkComboBoxText *MonitorButton = NULL;
 
 static int logo[3];
 static int flip[3];
 
 static int iconified = 0;
+static int Nmonitors;
+#define nsbuffer 1024
+static char sbuffer[nsbuffer];
 
 static void   handle_css(void);
 static void   init_ids(void);
@@ -84,6 +95,7 @@ static float  scale_to_speedfactor(float scale);
 static void   init_pixmap(GtkWidget *id, int n, int flip);
 static int    do_animate(void *dummy);
 static int    do_test_iconified(void *dummy);
+static void   doallworkspaces(void);
 static gboolean callback_func(GtkWidget *widget,
       GdkEventWindowState *event,
       gpointer user_data);
@@ -163,6 +175,9 @@ void init_ids()
    id_bg_color       = GTK_WIDGET(gtk_builder_get_object(builder, "id-bg-color"));
    id_bubbles        = GTK_WIDGET(gtk_builder_get_object(builder, "id-bubbles"));
    id_bubblesvalue   = GTK_WIDGET(gtk_builder_get_object(builder, "id-bubblesvalue"));
+
+   id_screen         = GTK_WIDGET(gtk_builder_get_object(builder, "id-Screen"));
+   id_allworkspaces  = GTK_WIDGET(gtk_builder_get_object(builder, "id-AllWorkspaces"));
 }
 
 int do_animate(void *dummy)
@@ -247,6 +262,9 @@ void init_pixmaps()
 
 void set_buttons()
 {
+   if (nomenu)
+      return;
+
    int h = human_action;
    human_action = 0;
 
@@ -272,6 +290,9 @@ void set_buttons()
    gdk_rgba_parse(&color, bgcolorstring);
    gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(id_bg_color),&color);
 
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(id_allworkspaces),allworkspaces);
+   doallworkspaces();
+
    human_action = h;
 
    FlagsChanged = 1;
@@ -297,6 +318,7 @@ MODULE_EXPORT void button_bubbles(GtkWidget *w)
       return;
    float value = gtk_range_get_value(GTK_RANGE(w));
    P("bubbles: %f %d\n",value,blimit);
+   remove_all_bubbles();
    create_bubbles(scale_to_bubblesfactor(value));
    FlagsChanged = 1;
 
@@ -363,6 +385,48 @@ MODULE_EXPORT void button_bg_color(GtkWidget *w)
    FlagsChanged = 1;
 }
 
+MODULE_EXPORT void button_allworkspaces(GtkWidget *w)
+{
+   (void) w;
+   if (!human_action)
+      return;
+   allworkspaces = ! allworkspaces;
+   doallworkspaces();
+   P("allworkspaces:%d\n",allworkspaces);
+   FlagsChanged = 1;
+}
+
+void doallworkspaces()
+{
+   P("allworkspaces: %d\n",allworkspaces);
+   if (allworkspaces)
+   {
+      gtk_window_stick(GTK_WINDOW(hauptfenster));
+      gtk_window_stick(GTK_WINDOW(gtkwin));
+   }
+   else
+   {
+      gtk_window_unstick(GTK_WINDOW(hauptfenster));
+      gtk_window_unstick(GTK_WINDOW(gtkwin));
+   }
+}
+
+MODULE_EXPORT void combo_screen(GtkComboBoxText *combo, gpointer data)
+{
+   (void)data;
+   int num = gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
+   monitor = num-1;
+   P("combo_screen: monitor:%d\n",monitor);
+   set_monitor(monitor);
+   int n = flimit;
+   create_fishes(0);
+   create_fishes(n);
+   n = blimit;
+   create_bubbles(0);
+   create_bubbles(n);
+   FlagsChanged = 1;
+}
+
 FILE *openflagsfile(char *mode)
 {
    char *h = getenv("HOME");
@@ -442,6 +506,14 @@ void ReadFlags()
 	 bgcolorstring = strdup(rest);
 	 P("bcolorstring: %s\n",bgcolorstring);
       }
+      else if(!strcmp(flag,"allworkspaces"))
+      {
+	 allworkspaces = atoi(rest) != 0;
+      }
+      else if(!strcmp(flag,"monitor"))
+      {
+	 monitor = atoi(rest);
+      }
       lineno++;
       free(line);
       free(flag);
@@ -457,13 +529,15 @@ void WriteFlags()
       I("Cannot write $HOME/%s\n",FLAGSFILE);
       return;
    }
-   fprintf(f, "fishes      %d\n", flimit);
-   fprintf(f, "bubbles     %d\n", blimit);
+   fprintf(f, "fishes       %d\n", flimit);
+   fprintf(f, "bubbles      %d\n", blimit);
    if (bcolorstring && strlen(bcolorstring))
-      fprintf(f, "bc          %s\n", bcolorstring);
+      fprintf(f, "bc           %s\n", bcolorstring);
    if(bgcolorstring && strlen(bgcolorstring))
-      fprintf(f, "bgc         %s\n", bgcolorstring);
-   fprintf(f, "speed       %d\n", (int)(100*speedfactor));
+      fprintf(f, "bgc          %s\n", bgcolorstring);
+   fprintf(f, "speed        %d\n", (int)(100*speedfactor));
+   fprintf(f, "allworkspaces %d\n", allworkspaces ? 1 : 0);
+   fprintf(f, "monitor %d\n", monitor);
    fclose(f);
    init_pixmaps();
 }
@@ -558,41 +632,174 @@ gboolean callback_func(GtkWidget *widget,
    return TRUE;
 }
 
-void ui()
+void ui(int nomenu)
 {
    P("here is uiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii\n");
 
-   builder = gtk_builder_new_from_string (xfishtank_xml, -1);
-   gtk_builder_connect_signals (builder, builder);
-   hauptfenster  = GTK_WIDGET(gtk_builder_get_object(builder, "hauptfenster"));
-
+   if (!nomenu)
    {
-      g_signal_connect(G_OBJECT(hauptfenster),
-	    "window-state-event",
-	    G_CALLBACK(callback_func),
-	    NULL);
+      builder = gtk_builder_new_from_string (xfishtank_xml, -1);
+      gtk_builder_connect_signals (builder, builder);
+      hauptfenster  = GTK_WIDGET(gtk_builder_get_object(builder, "hauptfenster"));
+
+      {
+	 g_signal_connect(G_OBJECT(hauptfenster),
+	       "window-state-event",
+	       G_CALLBACK(callback_func),
+	       NULL);
+      }
+
+      hauptfenstersc  = gtk_widget_get_style_context(hauptfenster);
+
+      handle_css();
+      char wtitle[100];
+      wtitle[0] = 0;
+      strcat(wtitle,"XfishtanK");
+#ifdef HAVE_CONFIG_H
+      strcat(wtitle,"-");
+      strncat(wtitle,VERSION,99 - strlen(wtitle));
+#endif
+      gtk_window_set_title(GTK_WINDOW(hauptfenster),wtitle);
+      gtk_window_set_resizable(GTK_WINDOW(hauptfenster),False);
+      gtk_widget_show_all (hauptfenster);
+      g_signal_connect (GTK_WINDOW(hauptfenster), "delete-event", G_CALLBACK (gtk_main_quit), NULL);
+      g_signal_connect (GTK_WINDOW(hauptfenster), "destroy", G_CALLBACK (gtk_main_quit), NULL);
+
+      init_ids();
    }
 
-   hauptfenstersc  = gtk_widget_get_style_context(hauptfenster);
+   XineramaScreenInfo *xininfo = XineramaQueryScreens(Dpy,&Nmonitors);
+   if (xininfo == NULL)
+   {
+      I("No xinerama...\n");
+      HaveXinerama = 0;
+   }
+   else
+   {
+      HaveXinerama = 1;
+   }
 
-   handle_css();
-   char wtitle[100];
-   wtitle[0] = 0;
-   strcat(wtitle,"XfishtanK");
-#ifdef HAVE_CONFIG_H
-   strcat(wtitle,"-");
-   strncat(wtitle,VERSION,99 - strlen(wtitle));
-#endif
-   gtk_window_set_title(GTK_WINDOW(hauptfenster),wtitle);
-   gtk_window_set_resizable(GTK_WINDOW(hauptfenster),False);
-   gtk_widget_show_all (hauptfenster);
-   g_signal_connect (GTK_WINDOW(hauptfenster), "delete-event", G_CALLBACK (gtk_main_quit), NULL);
-   g_signal_connect (GTK_WINDOW(hauptfenster), "destroy", G_CALLBACK (gtk_main_quit), NULL);
+   //
+   // handle number of screens/monitors
+
+   P("monitor: %d\n",monitor);
+   set_monitor(monitor);
+   //GtkComboBoxText *MonitorButton;
+   if (!nomenu)
+   {
+      MonitorButton = GTK_COMBO_BOX_TEXT(gtk_builder_get_object(builder,"id-Monitor"));
+   }
+   if (Nmonitors < 2)
+   {
+      if(!nomenu)
+	 gtk_widget_set_sensitive(GTK_WIDGET(MonitorButton),FALSE);
+      monitor = -1;
+   }
+   if (monitor < -1)
+      monitor = -1;
+   if (monitor >= Nmonitors)
+      monitor = Nmonitors - 1;
+
+   if(!nomenu)
+   {
+      gtk_combo_box_text_remove_all(MonitorButton);
+      gtk_combo_box_text_append_text(MonitorButton,_("all monitors"));
+
+      char *s = NULL;
+      int p = 0;
+      for (int i=0; i<Nmonitors; i++)
+      {
+	 snprintf(sbuffer,nsbuffer,_("monitor %d"),i);
+	 s = (char*) realloc(s,p+1+strlen(sbuffer));
+	 assert(s);
+	 strcpy(&s[p],sbuffer);
+	 gtk_combo_box_text_append_text(MonitorButton,&s[p]);
+	 p += 1+strlen(sbuffer);
+      }
+
+      P("monitor %d\n",monitor);
+      gtk_combo_box_set_active(GTK_COMBO_BOX(MonitorButton),monitor+1);
+      g_signal_connect (MonitorButton,"changed",G_CALLBACK(combo_screen),NULL);
+   }
+
+   XFree(xininfo);
 
-   init_ids();
 
    init_pixmaps();
    set_buttons();
    g_timeout_add_full(G_PRIORITY_DEFAULT, 150,   do_animate        ,NULL, NULL);
 }
 
+void set_monitor(int m)
+{
+   if (!HaveXinerama) return;
+   P("set_monitor: %d\n",m);
+   if(MonitorButton)
+      gtk_combo_box_set_active(GTK_COMBO_BOX(MonitorButton),m+1);
+   int x,y;
+   int n = xinerama(Dpy, 0,NULL,NULL,NULL,NULL);
+   if (m > n)
+      m = n;
+   if (m < -1)
+      m = -1;
+   monitor  = m;
+   if (monitor == -1)
+   {
+      width = fullwidth;
+      height = fullheight;
+      x = 0;
+      y = 0;
+      monitor_x = x;
+      monitor_y = y;
+   }
+   else
+      xinerama(Dpy, monitor, &x, &y, &width, &height);
+   P("x y width height: %d %d %d %d\n",x,y,width,height);
+   P("gtkwin: %p\n",(void*)gtkwin);
+   monitor_x = x;
+   monitor_y = y;
+   if(xfishtank_trans)
+   {
+      gtk_widget_set_size_request(gtkwin,width,height);
+      gtk_window_resize(GTK_WINDOW(gtkwin), width, height);
+      gtk_window_move(GTK_WINDOW(gtkwin),x,y);
+      {
+	 int w,h,x,y;
+	 gtk_window_get_size(GTK_WINDOW(gtkwin),&w, &h);
+	 gtk_window_get_position(GTK_WINDOW(gtkwin),&x, &y);
+	 P("get size: %d %d %d %d\n",w,h,x,y);
+
+      }
+
+   }
+   else
+      if(0){
+	 P("xmoveresize: %d %d %d %d\n",x,y,width,height);
+	 unsigned long valuemask = CWOverrideRedirect;
+	 XSetWindowAttributes attributes;
+	 attributes.override_redirect = 1;
+	 XChangeWindowAttributes(Dpy,wid,valuemask,&attributes);
+	 XChangeWindowAttributes(Dpy,wid1,valuemask,&attributes);
+	 XResizeWindow(Dpy,wid,width,height);
+	 XResizeWindow(Dpy,wid1,width,height);
+	 XMoveWindow(Dpy,wid,x,y);
+	 XMoveWindow(Dpy,wid1,x,y);
+
+	 Window root_return;
+	 int x_return,y_return;
+	 unsigned int width_return, height_return,
+		      border_width_return,depth_return;
+	 XGetGeometry(Dpy,wid,&root_return,&x_return,&y_return,
+	       &width_return,&height_return,
+	       &border_width_return,&depth_return);
+	 P("wid  %d %d %d %d\n",x_return,y_return,width_return,height_return);
+	 XGetGeometry(Dpy,wid1,&root_return,&x_return,&y_return,
+	       &width_return,&height_return,
+	       &border_width_return,&depth_return);
+	 P("wid1 %d %d %d %d\n",x_return,y_return,width_return,height_return);
+
+	 XFlush(Dpy);
+      }
+
+}
+
