@@ -27,10 +27,13 @@
 
 extern int FlagsChanged;
 
-extern void ui(void);
+extern void ui(int nomenu);
 extern void set_buttons(void);
 extern void WriteFlags();
 extern void ReadFlags(void);
 extern void show_dialog(int type, const char *format, const char *text);
 extern void iconify(void);
+extern void set_monitor(int m);
+extern int HaveXinerama;
+extern int monitor_x, monitor_y;
 
